/**********************************************************************
*  2007 Microchip Technology Inc.
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Incorporated ("Microchip") retains all ownership and 
* intellectual property rights in the code accompanying this message and in all 
* derivatives hereto.  You may use this code, and any derivatives created by 
* any person or entity by or on your behalf, exclusively with Microchip,s 
* proprietary products.  Your acceptance and/or use of this code constitutes 
* agreement to the terms and conditions of this notice.
*
* CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS".  NO 
* WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
* TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIP,S 
* PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
*
* YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE, WHETHER 
* IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), 
* STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, SPECIAL, 
* PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, FOR COST OR EXPENSE OF 
* ANY KIND WHATSOEVER RELATED TO THE CODE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN 
* ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT 
* ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO 
* THIS CODE, SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO 
* HAVE THIS CODE DEVELOPED.
*
* You agree that you are solely responsible for testing the code and 
* determining its suitability.  Microchip has no obligation to modify, test, 
* certify, or support the code.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author          	Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Jatinder Gharoo 	10/30/08  First release of source file
* 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* HARDWARE USED: 
* PIC24HJ64GP506 
* 
* SOFTWARE USED: 
* MPLAB IDE v8.60
* MPLAB C Compiler for PIC24 MCU V3.25 
* 
* CAN CONFIGURATION USED:
* Bit Time = (Sync Segment + Propagation Delay + Phase Segment 1 + Phase Segment 2)=20TQ
* Phase Segment 1 = 8TQ
* Phase Segment 2 = 6Tq
* Propagation Delay = 5Tq
* Sync Segment = 1TQ
* ################ BIT RATE OF 100kBit #################
*	
* 
*************************************************************************************************/


/* Include the appropriate header (.h) file, depending on device used */

#if defined(__dsPIC33F__)
#include <p33Fxxxx.h>
#elif defined(__PIC24H__)
#include <p24hxxxx.h>
#endif

#include "ecan.h"
#include "delay.h"

/*****************************************************************************************************
Configuration bits: 
Use internal 7.37MHz oscillator 	
*****************************************************************************************************/
_FOSCSEL(0x0001) 
_FOSC(0x00E7)
_FWDT(FWDTEN_OFF)
_FPOR(FPWRT_PWR1)
_FICD(0x0043)
        
/*****************************************************************************************************
Function Prototypes  
*****************************************************************************************************/
void oscConfig(void);

/*****************************************************************************************************
Globals   
*****************************************************************************************************/
mID canTxMessage1;
mID canTxMessage2;


/* Define ECAN Message Buffers */
ECAN1MSGBUF ecan1msgBuf __attribute__((space(dma),aligned(ECAN1_MSG_BUF_LENGTH*16)));




/*****************************************************************************************************
main() function 
*****************************************************************************************************/
int main ( void )
{	
	/* Configure Oscillator Clock Source */
	oscConfig();
	
	/* initialise other modules */				
	initECAN();
	initDMAECAN();
	
	/* Enable ECAN1 Interrupt */     	
	IEC2bits.C1IE=1;	
	/* enable Transmit interrupt */
	C1INTEbits.TBIE=1;
	/* Enable Receive interrupt */
	C1INTEbits.RBIE=1;
	
	/* configure and send a message */
	canTxMessage1.message_type=CAN_MSG_DATA;
	//canTxMessage.message_type=CAN_MSG_RTR;
	canTxMessage1.frame_type=CAN_FRAME_EXT;
	//canTxMessage.frame_type=CAN_FRAME_STD;
	canTxMessage1.buffer=0;
	canTxMessage1.id=0x11111111;
	canTxMessage1.data[0]=0x11;
	canTxMessage1.data[1]=0x11;
	canTxMessage1.data[2]=0x11;
	canTxMessage1.data[3]=0x11;
	canTxMessage1.data[4]=0x11;
	canTxMessage1.data[5]=0x11;
	canTxMessage1.data[6]=0x11;
	canTxMessage1.data[7]=0x11;
	canTxMessage1.data_length=8;

	/* configure and send a message */
	canTxMessage2.message_type=CAN_MSG_DATA;
	//canTxMessage.message_type=CAN_MSG_RTR;
	canTxMessage2.frame_type=CAN_FRAME_EXT;
	//canTxMessage.frame_type=CAN_FRAME_STD;
	canTxMessage2.buffer=1;
	canTxMessage2.id=0x02222222;
	canTxMessage2.data[0]=0x22;
	canTxMessage2.data[1]=0x22;
	canTxMessage2.data[2]=0x22;
	canTxMessage2.data[3]=0x22;
	canTxMessage2.data[4]=0x22;
	canTxMessage2.data[5]=0x22;
	canTxMessage2.data[6]=0x22;
	canTxMessage2.data[7]=0x22;
	canTxMessage2.data_length=8;
	
	/* Delay for a second */
	Delay(Delay_1S_Cnt);
		
	/* send a CAN message */
	sendECAN(&canTxMessage1);
		
	while(1)
	{

		// delay for one second 
		Delay(1000);
		// send another message 
		sendECAN(&canTxMessage2);

	}
}

/*****************************************************************************************************
oscConfig() function 
*****************************************************************************************************/
void oscConfig(void){

	/*  Configure Oscillator to operate the device at 40MIPS
 	Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
 	Fosc= 8M*40/(2*2)=80Mhz for 8M input clock */

 	CLKDIVbits.PLLPRE 	= 1;			/*  7,370MHz / 3    = 2,4567MHz   [0,8..8MHz]   */
	PLLFBDbits.PLLDIV	= 63;			/*  2,4567MHz* 65   = 159,683MHz  [100..200MHz] */
	CLKDIVbits.PLLPOST	= 0;			/*  159,683MHz/ 2   = 79,842MHz   [12.5..80MHz] */
	
	/* Disable Watch Dog Timer */
	RCONbits.SWDTEN=0;

	/* Wait for PLL to lock */
	while(OSCCONbits.LOCK!=1) {};
}

/****** START OF INTERRUPT SERVICE ROUTINES *********/

/* Replace the interrupt function names with the    */
/* appropriate names depending on interrupt source. */

/* The names of various interrupt functions for     */
/* each device are defined in the linker script.    */


/* Interrupt Service Routine 1                      */
/* No fast context save, and no variables stacked   */
void __attribute__((interrupt,no_auto_psv))_C1Interrupt(void)  
{
	/* check to see if the interrupt is caused by receive */     	 
    if(C1INTFbits.RBIF)
    {
		C1INTFbits.RBIF = 0;
	}
	else if(C1INTFbits.TBIF)
    {

	    /* clear flag */
		C1INTFbits.TBIF = 0;	    
	}
	else;
	
	/* clear interrupt flag */
	IFS2bits.C1IF=0;
}

/********* END OF INTERRUPT SERVICE ROUTINES ********/
